package de.intevation.mxd.reader;

import java.lang.Exception;

import org.apache.log4j.Logger;

import com.esri.arcgis.carto.IFeatureRenderer;
import com.esri.arcgis.carto.ClassBreaksRenderer;
import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.SimpleMarkerSymbol;
import com.esri.arcgis.display.SimpleFillSymbol;

import org.w3c.dom.Element;

import de.intevation.mxd.utils.MapToXMLUtils;

/**
 * Reads the ClassBreaksRender Object Attributes.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class ClassBreakRendererReader
implements IRendererReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(ClassBreakRendererReader.class);

    /**
     * Private member.
     */
    private ClassBreaksRenderer renderer;
    private Element layer;
    private MapToXMLUtils util;


    public ClassBreakRendererReader(IFeatureRenderer renderer)
    throws Exception {
        logger.debug("contructor()");
        if(renderer instanceof ClassBreaksRenderer)
            this.renderer = (ClassBreaksRenderer)renderer;
        else
            throw new Exception("Not a ClassBreaksRenderer!");
    }

    /**
     * Setter for the parent XML element.
     *
     * @param parent The XML parent node.
     */
    public void setParent(Element parent) {
        this.layer = parent;
    }

    /**
     * Setter for XML document helper.
     *
     * @param util The helper class for storing map information.
     */
    public void setUtil(MapToXMLUtils util) {
        this.util = util;
    }

    /**
     * Reads the Renderer attributes.
     *
     * @return The XML node.
     */
    public Element read()
    throws Exception {
        logger.debug("read()");

        for(int i = 0; i < renderer.getBreakCount(); i++){
            Element rendererElement = util.addRenderer(layer);
            rendererElement.setAttribute("value",
                String.valueOf(renderer.getBreak(i)));
            rendererElement.setAttribute(
                "field_count",
                String.valueOf(renderer.getFieldCount()));
            for(int j = 0; j < renderer.getFieldCount(); j++) {
                rendererElement.setAttribute(
                    "expression_field_" + j,
                    renderer.getField(j));

            }
            rendererElement.setAttribute(
                "min_value",
                String.valueOf(renderer.getMinimumBreak()));
            rendererElement.setAttribute("expression_operator", "<=");

            ISymbol sym = renderer.getSymbol(i);
            MarkerSymbolReader markerReader = new MarkerSymbolReader();
            LineSymbolReader lineReader = new LineSymbolReader();
            FillSymbolReader fillReader = new FillSymbolReader();
            if(markerReader.canRead(sym)) {
                markerReader.setSymbol(sym);
                markerReader.setUtil(util);
                markerReader.setParent(rendererElement);
                markerReader.read();
            }
            else if(lineReader.canRead(sym)) {
                lineReader.setSymbol(sym);
                lineReader.setUtil(util);
                lineReader.setParent(rendererElement);
                lineReader.read();
            }
            else if(fillReader.canRead(sym)) {
                fillReader.setSymbol(sym);
                fillReader.setUtil(util);
                fillReader.setParent(rendererElement);
                fillReader.read();
            }
            else {
                logger.debug("Not a known symbol type: " +
                             sym.getClass().toString());
            }
        }
        return layer;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
